@@ -5,21 +5,40 @@ BEGIN { push(@INC, ".."); };
 use WebminCore;
 &init_config();
 
+# check_exports()
+# Returns an error message if the NFS exports package is missing
+sub check_exports
+{
+if ($gconfig{'os_type'} eq 'freebsd') {
+	# Check for commands on FreeBSD
+	foreach my $c ("mountd", "nfsd") {
+		&has_command($c) || return &text('check_ecmd', $c);
+		}
+	return undef;
+	}
+else {
+	# Don't know
+	return undef;
+	}
+}
+
 # list_exports()
 # Returns the current exports list
 sub list_exports
 {
 local(@rv, $lnum, $_);
-open(EXP, $config{'exports_file'});
+open(EXP, "<".$config{'exports_file'});
 $lnum = -1; $index = 0;
 while(<EXP>) {
 	$lnum++;
 	s/\r|\n//g;	# remove newlines
 	s/#.*$//g;	# remove comments
 	next if (!/\S/);
+	s/\\ /<sp>/g;	# hack to support splitting on space
 	local @w = split(/[\s=]+/, $_);
 	local %exp;
 	for($i=0; $i<@w; $i++) {
+		$w[$i] =~ s/<sp>/ /g;
 		if ($w[$i] =~ /^\//) { push(@{$exp{'dirs'}}, $w[$i]); }
 		elsif ($w[$i] eq "-maproot") { $exp{'maproot'} = $w[++$i]; }
 		elsif ($w[$i] eq "-r") { $exp{'maproot'} = $w[++$i]; }
@@ -67,6 +86,9 @@ sub delete_export
 sub export_line
 {
 local %exp = %{$_[0]};
+foreach my $d (@{$exp{'dirs'}}) {
+	$d =~ s/ /\\ /g;
+	}
 local $rv = join(' ', @{$exp{'dirs'}});
 if ($exp{'alldirs'}) { $rv .= " -alldirs"; }
 if ($exp{'ro'}) { $rv .= " -ro"; }
