@@ -309,8 +309,7 @@ elsif ($fcron) {
 			 undef, $cron_temp_file, undef);
 	}
 else {
-	system("cp ".quotemeta(
-			&translate_filename("$config{'cron_dir'}/$_[0]->{'user'}")).
+	system("cp ".&translate_filename("$config{'cron_dir'}/$_[0]->{'user'}").
 	       " ".quotemeta($cron_temp_file)." 2>/dev/null");
 	}
 }
@@ -514,7 +513,7 @@ if (&read_file_contents($cron_temp_file) =~ /\S/) {
 		$ENV{"VISUAL"} = $ENV{"EDITOR"} =
 			"$module_root_directory/cron_editor.pl";
 		$ENV{"CRON_EDITOR_COPY"} = $cron_temp_file;
-		system("chown ".quotemeta($_[0])." ".quotemeta($cron_temp_file));
+		system("chown $_[0] $cron_temp_file");
 		local $oldpwd = &get_current_dir();
 		chdir("/");
 		if ($single_user) {
@@ -601,7 +600,7 @@ sub user_sub
 {
 local($tmp);
 $tmp = $_[0];
-$tmp =~ s/USER/quotemeta($_[1])/ge;
+$tmp =~ s/USER/$_[1]/g;
 return $tmp;
 }
 
@@ -1558,7 +1557,7 @@ if ($err) {
 
 =head2 cleanup_temp_files
 
-Called from cron to delete old files in the Webmin temp directory, and also
+Called from cron to delete old files in the Webmin /tmp directory, and also
 old lock links directories.
 
 =cut
@@ -1570,7 +1569,7 @@ if (!$gconfig{'tempdelete_days'}) {
 	return;
 	}
 
-# Cleanup files in the default Webmin temp directory
+# Cleanup files in /tmp/.webmin
 if ($gconfig{'tempdir'} && !$gconfig{'tempdirdelete'}) {
 	print STDERR "Temp file clearing is not done for the custom directory $gconfig{'tempdir'}\n";
 	}
@@ -1578,7 +1577,7 @@ else {
 	my $tempdir = &transname();
 	$tempdir =~ s/\/([^\/]+)$//;
 	if (!$tempdir || $tempdir eq "/") {
-		$tempdir = &default_webmin_temp_dir();
+		$tempdir = "/tmp/.webmin";
 		}
 
 	my $cutoff = time() - $gconfig{'tempdelete_days'}*24*60*60;
