@@ -1,13 +1,18 @@
 # log_parser.pl
 # Functions for parsing this module's logs
 
+use strict;
+use warnings;
+no warnings 'redefine';
+no warnings 'uninitialized';
 do 'exports-lib.pl';
+our (%text);
 
 # parse_webmin_log(user, script, action, type, object, &params, [long])
 # Converts logged information from this module into human-readable form
 sub parse_webmin_log
 {
-local ($user, $script, $action, $type, $object, $p, $long) = @_;
+my ($user, $script, $action, $type, $object, $p, $long) = @_;
 $p->{'host'} = $p->{'host'} ? &html_escape($p->{'host'}) : '*';
 $object = &html_escape($object);
 if ($type eq 'exports') {
