@@ -633,16 +633,16 @@ else {
 	}
 }
 
-# read_file_lines_as_user(file, [read-only])
+# read_file_lines_as_user(file, ...)
 sub read_file_lines_as_user
 {
-local ($file, $ro) = @_;
+local @args = @_;
 if (&should_switch_user($file)) {
 	return &eval_as_unix_user(
-		$access{'user'}, sub { &read_file_lines($file, $ro) });
+		$access{'user'}, sub { &read_file_lines(@args) });
 	}
 else {
-	return &read_file_lines($file, $ro);
+	return &read_file_lines(@args);
 	}
 }
 
