@@ -66,12 +66,8 @@ local @links;
 foreach $l ("tree", "user", "size", "cpu", ($has_zone ? ("zone") : ()),
 	    "search", "run") {
 	next if ($l eq "run" && !$access{'run'});
-	local $link;
-	if ($l ne $_[0]) { $link .= "<a href=index_$l.cgi>"; }
-	else { $link .= "<b>"; }
-	$link .= $text{"index_$l"};
-	if ($l ne $_[0]) { $link .= "</a>"; }
-	else { $link .= "</b>"; }
+	my $link = ( $l ne $_[0] ? &ui_link("index_".$l.".cgi", $text{"index_$l"}) :
+		"<b onclick='".&ui_page_refresh()."' style='cursor: pointer'>".$text{"index_$l"}."</b>" );
 	push(@links, $link);
 	}
 print &ui_links_row(\@links);
@@ -128,7 +124,7 @@ if ($gconfig{'os_type'} eq 'windows') {
 	open(TEMP, ">$temp");
 	print TEMP $_[4];
 	close(TEMP);
-	&open_execute_command(OUT, "$_[0] <$temp 2>&1", 1);
+	&open_execute_command(OUT, "$_[0] <".quotemeta($temp)." 2>&1", 1);
 	local $fh = $_[3];
 	while(<OUT>) {
 		if ($_[5]) {
@@ -160,14 +156,14 @@ else {
 		close(OUTr); close(INw);
 
 		if ($_[1]) {
+			local @u = getpwuid($_[1]);
 			if (defined($_[2])) {
 				# switch to given UID and GID
 				&switch_to_unix_user(
-					[ undef, undef, $_[1], $_[2] ]);
+					[ $u[0], undef, $_[1], $_[2] ]);
 				}
 			else {
 				# switch to UID and all GIDs
-				local @u = getpwuid($_[1]);
 				&switch_to_unix_user(\@u);
 				}
 			}
@@ -175,6 +171,7 @@ else {
 		# run the command
 		delete($ENV{'FOREIGN_MODULE_NAME'});
 		delete($ENV{'SCRIPT_NAME'});
+		chdir(&tempname_dir()) if (&get_current_dir() =~ /$root_directory/);
 		exec("/bin/sh", "-c", $_[0]);
 		print "Exec failed : $!\n";
 		exit 1;
@@ -198,7 +195,7 @@ else {
 		local $sel = select($rmask, undef, undef, 1);
 		if ($sel > 0 && vec($rmask, $fn, 1)) {
 			# got something to read.. print it
-			sysread(OUTr, $buf, 1024) || last;
+			sysread(OUTr, $buf, &get_buffer_size()) || last;
 			$got += length($buf);
 			if ($_[5]) {
 				$buf = &html_escape($buf);
@@ -252,11 +249,11 @@ do {    local $oldexit = $?;
 	} while($xp > 0);
 }
 
-# pty_process_exec(command, [uid, gid])
+# pty_process_exec(command, [uid, gid], [force-binary-name])
 # Starts the given command in a new pty and returns the pty filehandle and PID
 sub pty_process_exec
 {
-local ($cmd, $uid, $gid) = @_;
+local ($cmd, $uid, $gid, $binary) = @_;
 if (&is_readonly_mode()) {
 	# When in readonly mode, don't run the command
 	$cmd = "/bin/true";
@@ -264,17 +261,19 @@ if (&is_readonly_mode()) {
 &webmin_debug_log('CMD', "cmd=$cmd uid=$uid gid=$gid")
 	if ($gconfig{'debug_what_cmd'});
 
+my ($ptyfh, $ttyfh, $pty, $tty, $TIOCSCTTY);
+
 eval "use IO::Pty";
 if (!$@) {
 	# Use the IO::Pty perl module if installed
-	local $ptyfh = new IO::Pty;
+	$ptyfh = new IO::Pty;
 	if (!$ptyfh) {
 		&error("Failed to create new PTY with IO::Pty");
 		}
 	local $pid = fork();
 	if (!$pid) {
-		local $ttyfh = $ptyfh->slave();
-		local $tty = $ptyfh->ttyname();
+		$ttyfh = $ptyfh->slave();
+		$tty = $ptyfh->ttyname();
 		if (defined(&close_controlling_pty)) {
 			&close_controlling_pty();
 			}
@@ -289,8 +288,9 @@ if (!$@) {
 
 		close(STDIN); close(STDOUT); close(STDERR);
 		untie(*STDIN); untie(*STDOUT); untie(*STDERR);
-		if ($_[1]) {
-			&switch_to_unix_user([ undef, undef, $_[1], $_[2] ]);
+		if ($uid) {
+			my $username = getpwuid($uid);
+			&switch_to_unix_user([ $username, undef, $uid, $gid ]);
 			}
 
 		close($ptyfh);		# Used by other side only
@@ -298,16 +298,85 @@ if (!$@) {
 		open(STDOUT, ">&".fileno($ttyfh));
 		open(STDERR, ">&".fileno($ttyfh));
 		close($ttyfh);		# Already dup'd
-		exec($cmd);
-		print "Exec failed : $!\n";
+		if ($binary) {
+			my @args = &split_quoted_string($cmd);
+			my $args0 = $args[0];
+			$args[0] = $binary;
+			exec { $args0 } @args;
+			}
+		else {
+			exec($cmd);
+			}
+		print STDERR "Exec failed : $!\n";
 		exit 1;
 		}
 	$ptyfh->close_slave();
 	return ($ptyfh, $pid);
 	}
+elsif (defined &linux_openpty &&
+       do { ($ptyfh, $ttyfh, $tty, $TIOCSCTTY) = linux_openpty(); $ptyfh }) {
+	# Use pure-Perl Linux fallback
+	my $pid = fork();
+	if (!$pid) {
+		if (defined(&close_controlling_pty)) {
+			&close_controlling_pty();
+			}
+		my $need_ctty = $ENV{'TERM'} ? 1 : 0;
+		setsid()                            # create new session group
+			or do {
+				# don't fail, just log it if we have a terminal
+				&error_stderr("setsid failed: $!")
+					if ($need_ctty);
+				};
+		close($ptyfh);                      # child must not hold master
+		my $ctty_arg = 0;                   # try normal attach
+		ioctl($ttyfh, $TIOCSCTTY, $ctty_arg) or do {
+			$ctty_arg = 1;              # try force attach
+			ioctl($ttyfh, $TIOCSCTTY, $ctty_arg)
+				or do {
+					# don't fail, just log it if we have a
+					# terminal
+					&error_stderr("TIOCSCTTY failed: $!")
+						if ($need_ctty);
+					}
+			};
+
+		# Turn off echoing, if we can
+		eval "use IO::Stty";
+		if (!$@) {
+			IO::Stty::stty($ttyfh, 'raw', '-echo');
+			}
+
+		close(STDIN); close(STDOUT); close(STDERR);
+		untie(*STDIN); untie(*STDOUT); untie(*STDERR);
+		if ($uid) {
+			my $username = getpwuid($uid);
+			&switch_to_unix_user([ $username, undef, $uid, $gid ]);
+		}
+
+		open(STDIN, "<&".fileno($ttyfh));
+		open(STDOUT, ">&".fileno($ttyfh));
+		open(STDERR, ">&".fileno($ttyfh));
+		close($ttyfh);
+
+		if ($binary) {
+			my @args = &split_quoted_string($cmd);
+			my $args0 = $args[0];
+			$args[0] = $binary;
+			exec { $args0 } @args;
+			}
+		else {
+			exec($cmd);
+			}
+		print STDERR "Exec failed : $!\n";
+		exit 1;
+		}
+	close($ttyfh);
+	return ($ptyfh, $pid);
+	}
 else {
 	# Need to create a PTY using built-in Webmin code
-	local ($ptyfh, $ttyfh, $pty, $tty) = &get_new_pty();
+	($ptyfh, $ttyfh, $pty, $tty) = &get_new_pty();
 	$tty || &error("Failed to create new PTY - try installing the IO::Tty Perl module");
 	local $pid = fork();
 	if (!$pid) {
@@ -340,16 +409,25 @@ else {
 		close(STDIN); close(STDOUT); close(STDERR);
 		untie(*STDIN); untie(*STDOUT); untie(*STDERR);
 		#setpgrp(0, $$);
-		if ($_[1]) {
-			&switch_to_unix_user([ undef, undef, $_[1], $_[2] ]);
+		if ($uid) {
+			my $username = getpwuid($uid);
+			&switch_to_unix_user([ $username, undef, $uid, $gid ]);
 			}
 
 		open(STDIN, "<$tty");
 		open(STDOUT, ">&$ttyfh");
 		open(STDERR, ">&STDOUT");
 		close($ptyfh);
-		exec($cmd);
-		print "Exec failed : $!\n";
+		if ($binary) {
+			my @args = &split_quoted_string($cmd);
+			my $args0 = $args[0];
+			$args[0] = $binary;
+			exec { $args0 } @args;
+			}
+		else {
+			exec($cmd);
+			}
+		print STDERR "Exec failed : $!\n";
 		exit 1;
 		}
 	close($ttyfh);
@@ -381,13 +459,15 @@ $has_lsof_command = &has_command("lsof");
 sub find_socket_processes
 {
 local @rv;
-open(LSOF, "lsof -i '$_[0]:$_[1]' |");
-while(<LSOF>) {
-	if (/^(\S+)\s+(\d+)/) {
-		push(@rv, $2);
+if ($has_lsof_command) {
+	open(LSOF, "$has_lsof_command -i ".quotemeta("$_[0]:$_[1]")." |");
+	while(<LSOF>) {
+		if (/^(\S+)\s+(\d+)/) {
+			push(@rv, $2);
+			}
 		}
+	close(LSOF);
 	}
-close(LSOF);
 return @rv;
 }
 
@@ -396,13 +476,54 @@ return @rv;
 sub find_ip_processes
 {
 local @rv;
-open(LSOF, "lsof -i '\@$_[0]' |");
-while(<LSOF>) {
-	if (/^(\S+)\s+(\d+)/) {
-		push(@rv, $2);
+if ($has_lsof_command) {
+	open(LSOF, "$has_lsof_command -i ".quotemeta("\@$_[0]")." |");
+	while(<LSOF>) {
+		if (/^(\S+)\s+(\d+)/) {
+			push(@rv, $2);
+			}
 		}
+	close(LSOF);
+	}
+return @rv;
+}
+
+# find_all_process_sockets()
+# Returns all network connections made by any process
+sub find_all_process_sockets
+{
+local @rv;
+if ($has_lsof_command) {
+	open(LSOF, "$has_lsof_command -i tcp -i udp -n |");
+	while(<LSOF>) {
+		if (/^(\S+)\s+(\d+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+).*(TCP|UDP)\s+(.*)/) {
+			local $n = { 'cmd' => $1,
+				     'pid' => $2,
+				     'fd' => $4,
+				     'type' => $5,
+				     'proto' => $7 };
+			local $m = $8;
+			if ($m =~ /^([^:\s]+):([^:\s]+)\s+\(listen\)/i) {
+				$n->{'lhost'} = $1;
+				$n->{'lport'} = $2;
+				$n->{'listen'} = 1;
+				}
+			elsif ($m =~ /^([^:\s]+):([^:\s]+)->([^:\s]+):([^:\s]+)\s+\((\S+)\)/) {
+				$n->{'lhost'} = $1;
+				$n->{'lport'} = $2;
+				$n->{'rhost'} = $3;
+				$n->{'rport'} = $4;
+				$n->{'state'} = $5;
+				}
+			elsif ($m =~ /^([^:\s]+):([^:\s]+)/) {
+				$n->{'lhost'} = $1;
+				$n->{'lport'} = $2;
+				}
+			push(@rv, $n);
+			}
+		}
+	close(LSOF);
 	}
-close(LSOF);
 return @rv;
 }
 
@@ -410,62 +531,51 @@ return @rv;
 # Returns all network connections made by some process
 sub find_process_sockets
 {
-local @rv;
-open(LSOF, "lsof -i tcp -i udp -n |");
-while(<LSOF>) {
-	if (/^(\S+)\s+(\d+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+).*(TCP|UDP)\s+(.*)/
-	    && $2 eq $_[0]) {
-		local $n = { 'fd' => $4,
-			     'type' => $5,
-			     'proto' => $7 };
-		local $m = $8;
-		if ($m =~ /^([^:\s]+):([^:\s]+)\s+\(listen\)/i) {
-			$n->{'lhost'} = $1;
-			$n->{'lport'} = $2;
-			$n->{'listen'} = 1;
-			}
-		elsif ($m =~ /^([^:\s]+):([^:\s]+)->([^:\s]+):([^:\s]+)\s+\((\S+)\)/) {
-			$n->{'lhost'} = $1;
-			$n->{'lport'} = $2;
-			$n->{'rhost'} = $3;
-			$n->{'rport'} = $4;
-			$n->{'state'} = $5;
-			}
-		elsif ($m =~ /^([^:\s]+):([^:\s]+)/) {
-			$n->{'lhost'} = $1;
-			$n->{'lport'} = $2;
-			}
-		push(@rv, $n);
-		}
-	}
-close(LSOF);
-return @rv;
+my ($pid) = @_;
+return grep { $_->{'pid'} == $pid } &find_all_process_sockets();
 }
 
-# find_process_files(pid)
+# find_all_process_files()
+# Returns all files currently held open by all processes
+sub find_all_process_files
+{
+return &find_process_files();
+}
+
+# find_process_files([pid])
 # Returns all files currently held open by some process
 sub find_process_files
 {
+local ($pid) = @_;
 local @rv;
-open(LSOF, "lsof -p '$_[0]' |");
-while(<LSOF>) {
-	if (/^(\S+)\s+(\d+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\d+),(\d+)\s+(\d+)\s+(\d+)\s+(.*)/) {
-		push(@rv, { 'fd' => lc($4),
-			    'type' => lc($5),
-			    'device' => [ $6, $7 ],
-			    'size' => $8,
-			    'inode' => $9,
-			    'file' => $10 });
+if ($has_lsof_command) {
+	if (defined($pid)) {
+		open(LSOF, "$has_lsof_command -p ".quotemeta($pid)." |");
 		}
-	elsif (/^(\S+)\s+(\d+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\d+),(\d+)\s+(\d+)\s+(.*)/) {
-		push(@rv, { 'fd' => lc($4),
-			    'type' => lc($5),
-			    'device' => [ $6, $7 ],
-			    'inode' => $8,
-			    'file' => $9 });
+	else {
+		open(LSOF, "$has_lsof_command |");
 		}
+	while(<LSOF>) {
+		if (/^(\S+)\s+(\d+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\d+),(\d+)\s+(\d+)\s+(\d+)\s+(.*)/) {
+			push(@rv, { 'pid' => $2,
+				    'fd' => lc($4),
+				    'type' => lc($5),
+				    'device' => [ $6, $7 ],
+				    'size' => $8,
+				    'inode' => $9,
+				    'file' => $10 });
+			}
+		elsif (/^(\S+)\s+(\d+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\d+),(\d+)\s+(\d+)\s+(.*)/) {
+			push(@rv, { 'pid' => $2,
+				    'fd' => lc($4),
+				    'type' => lc($5),
+				    'device' => [ $6, $7 ],
+				    'inode' => $8,
+				    'file' => $9 });
+			}
+		}
+	close(LSOF);
 	}
-close(LSOF);
 return @rv;
 }
 
@@ -533,11 +643,14 @@ else {
 	}
 }
 
-# can_view_process(user)
+# can_view_process(&process)
 # Returns 1 if processes belong to this user can be seen
 sub can_view_process
 {
-local ($user) = @_;
+local ($p) = @_;
+return 0 if ($p->{'pid'} == $$ && $config{'hide_self'});
+return 0 if ($p->{'_pscmd'} && $config{'hide_self'});
+local $user = $p->{'user'};
 if ($hide{$user}) {
 	return 0;
 	}
@@ -644,5 +757,45 @@ else {
 	}
 }
 
+# format_stime(&proc)
+# Returns the process start time in human-readable format
+sub format_stime
+{
+my ($p) = @_;
+if (!$p->{'_stime_unix'}) {
+	return $p->{'_stime'}
+	}
+elsif (time() - $p->{'_stime_unix'} > 86400) {
+	return &make_date($p->{'_stime_unix'}, 1);
+	}
+else {
+	return &make_date($p->{'_stime_unix'});
+	}
+}
+
+# count_processes()
+sub count_processes
+{
+my $process_count = 0;
+if ($gconfig{'os_type'} eq 'windows') {
+	open(my $ps, '-|', 'tasklist /FO CSV') || return -1;
+	while (my $line = <$ps>) {
+		next if $. == 1;  # Skip the header line
+		$process_count++;
+		}
+	close($ps);
+	return $process_count;
+	}
+else {
+	open(my $ps, '-|', 'ps -e') || return -1;
+	while (<$ps>) {
+		$process_count++;
+		}
+	close($ps);
+	# Skip the header line
+	return $process_count - 1;
+	}
+}
+
 1;
 
